/*********************************************************************************

   PROJECT:			CVDRIK
   FILE:			CVDRIKIPFeature.cpp
   AUTHOR:			PRITHWIJIT GUHA
   DATE:
   DESCRIPTION:		CPP File for Feature Extraction Functions

*********************************************************************************/


/*************************** OTHER INCLUDE FILES *****************************/

#include "CVDRIKImageProcessing.h"

#define EPSILON 0.0001
#define PI 3.141593


/*****************************************************************************/


/******************************************************************************

                                FUNCTIONS

******************************************************************************/


// Function for Color Negative Image Computation
// Inputs :-
//			(IplImage*) rgbImg : Pointer to the 3-Channel RGB Image
//			(IplImage*) negImg : Pointer to the 3-Channel Negative Image
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = computeNegativeImage( &rgbImg , &negImg );
unsigned char computeNegativeImage( IplImage* rgbImg , IplImage* negImg )
{
	// Check Inputs
	if( ( rgbImg->imageSize <= 0 ) || ( rgbImg->nChannels != 3 ) || ( negImg->imageSize <= 0 ) || ( negImg->nChannels != 3 ) )
	{
		// Error Code 1 : Invalid Input
		return( 1 );
	}
	
	// Check Data Size Mismatch
	if( ( rgbImg->width != negImg->width ) || ( rgbImg->height != negImg->height ) )
	{
		// Error Code 5 : Data Size Mismatch
		return( 5 );
	}
	
	// Set the Image Pixels to ZERO Intensity Values
	cvZero( negImg );
	
	// Computing the Color Negation Over Pixels
	int baseIndx = -( rgbImg->widthStep );
	for( int y = 0 ; y < ( rgbImg->height ) ; ++y )
	{
		// Update the Base Indices
		baseIndx = baseIndx + ( rgbImg->widthStep );
		
		// Set the Current Indices
		int currIndx = baseIndx - 3;
		
		// Looping Over the Columns
		for( int x = 0 ; x < ( rgbImg->width ) ; ++x )
		{
			// Update the Current Indices
			currIndx = currIndx + 3;
			
			// Computing the Color Negation
			// NOTE : Typecast the Data to "unsigned char" while Reading from the Image Array
			// NOTE : Typecast the Data to "char" while Assigning to the Image Array
			int blue = (unsigned char) rgbImg->imageData[ currIndx ];
			negImg->imageData[ currIndx ] = (char) ( 255 - blue );
			
			int green = (unsigned char) rgbImg->imageData[ currIndx + 1 ];
			negImg->imageData[ currIndx ] = (char) ( 255 - green );
			
			int red = (unsigned char) rgbImg->imageData[ currIndx + 2 ];
			negImg->imageData[ currIndx + 2 ] = (char) ( 255 - red );
		}
	}
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}





